import typer

from .cli import cmd_create, verbose


def main():
    app = typer.Typer(short_help="Utility to manage a pypy.org metadata cache")

    app.command()(cmd_create.create)
    app.callback()(verbose.verbose)
    app()


if __name__ == "__main__":
    main()



import typer
from metadict import MetaDict

state = MetaDict({"verbose": False})


def verbose(
    verbose: bool = typer.Option(False, "--verbose", "-v", help="Enable verbose mode")
):
    if verbose:
        state["verbose"] = True



import os
from tempfile import mkdtemp

from .mount import MS_BIND, MS_PRIVATE, MS_REC, mount
from .tar import extract_tar
from .unshare import set_user_level_root

STD_MOUNTS = [
    ("/proc", "proc/", None, MS_BIND | MS_REC),
    ("/dev", "dev/", None, MS_BIND | MS_REC),
    ("/sys", "sys/", None, MS_BIND | MS_REC),
    ("tmpfs", "run/", "tmpfs"),
]


def prepare_rootfs(rootfs_filename: str):
    set_user_level_root()
    mount(None, "/", None, MS_REC | MS_PRIVATE)
    mount_dir = mkdtemp()
    mount("tmpfs", mount_dir, "tmpfs")
    os.chdir(mount_dir)
    for mount_args in STD_MOUNTS:
        os.makedirs(mount_args[1])
        mount(*mount_args)
    extract_tar(rootfs_filename, mount_dir)
    if os.path.exists(f"{mount_dir}/etc/resolv.conf"):
        os.unlink(f"{mount_dir}/etc/resolv.conf")
    with open(f"{mount_dir}/etc/resolv.conf", "w") as f:
        f.write("")
    mount("/etc/resolv.conf", f"{mount_dir}/etc/resolv.conf", None, MS_BIND),



import os

import typer

from ..distros import URL_MAP
from ..download import download
from ..rootfs import prepare_rootfs


def check_distro_name(distro_name: str):
    return distro_name


def create(
    distro_name: str = typer.Argument(...),
):
    distro_url = URL_MAP.get(distro_name)
    if not distro_url:
        raise typer.BadParameter(f"Unknown distro {distro_name}")

    typer.echo(f"Downloading file {distro_url} for {distro_name}")
    tar_fname = download(distro_url)
    print("Extracting rootfs... ", end="", flush=True)
    prepare_rootfs(tar_fname)
    print("\nChrooting into rootfs")
    os.chroot(".")
    os.execv("/bin/sh", ["/bin/sh"])
